/****************************************************************************
 * This file is part of the project AqHome.
 * AqHome (c) by 2025 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "aqhome/msg/ipc/data/m_ipcd_multidata.h"
#include "aqhome/msg/ipc/m_ipc_tag16.h"
#include "aqhome/msg/ipc/data/m_ipcd.h"
#include "aqhome/msg/ipc/m_ipc.h"

#include <gwenhywfar/text.h>
#include <gwenhywfar/tag16.h>
#include <gwenhywfar/buffer.h>
#include <gwenhywfar/debug.h>



/* ------------------------------------------------------------------------------------------------
 * forward declarations
 * ------------------------------------------------------------------------------------------------
 */



/* ------------------------------------------------------------------------------------------------
 * implementation
 * ------------------------------------------------------------------------------------------------
 */

AQH_MESSAGE *AQH_IpcdMessageMultiData_new(uint16_t code, uint32_t msgId, uint32_t refMsgId, uint32_t flags,
                                          const AQH_VALUE *value, const uint64_t *i64Ptr, int numOfDataPoints)
{
  AQH_MESSAGE *msg;
  GWEN_BUFFER *buf;
  int rv;

  buf=GWEN_Buffer_new(0, 256, 0, 1);

  GWEN_Tag16_WriteUint32TagToBuffer(AQH_MSGDATA_MULTIDATA_TAGS_FLAGS, flags, buf);
  rv=AQH_Tag16_WriteValueAsTagToBuffer(AQH_MSGDATA_MULTIDATA_TAGS_VALUE, value, buf);
  if (rv<0) {
    DBG_INFO(AQH_LOGDOMAIN, "here (%d)", rv);
    GWEN_Buffer_free(buf);
    return NULL;
  }

  if (i64Ptr && numOfDataPoints)
    GWEN_Tag16_WriteTagToBuffer(AQH_MSGDATA_MULTIDATA_TAGS_DATA, (const uint8_t*)i64Ptr, numOfDataPoints*2*sizeof(uint64_t), buf);

  msg=AQH_IpcMessage_new(AQH_IPC_PROTOCOL_DATA_ID, AQH_IPC_PROTOCOL_DATA_VERSION, code, msgId, refMsgId,
                         GWEN_Buffer_GetUsedBytes(buf), (const uint8_t*) GWEN_Buffer_GetStart(buf));
  GWEN_Buffer_free(buf);
  return msg;
}



AQH_MESSAGE *AQH_IpcdMessageMultiData_newForOne(uint16_t code, uint32_t msgId, uint32_t refMsgId, uint32_t flags,
                                                const AQH_VALUE *value, uint64_t timeStamp, double dataPoint)
{
  AQH_MESSAGE *msg;
  GWEN_BUFFER *buf;
  int rv;
  union {double f; uint64_t i;} u;
  uint64_t arrayToSend[2];

  buf=GWEN_Buffer_new(0, 256, 0, 1);

  GWEN_Tag16_WriteUint32TagToBuffer(AQH_MSGDATA_MULTIDATA_TAGS_FLAGS, flags, buf);
  rv=AQH_Tag16_WriteValueAsTagToBuffer(AQH_MSGDATA_MULTIDATA_TAGS_VALUE, value, buf);
  if (rv<0) {
    DBG_INFO(AQH_LOGDOMAIN, "here (%d)", rv);
    GWEN_Buffer_free(buf);
    return NULL;
  }

  arrayToSend[0]=timeStamp;
  u.f=dataPoint;
  arrayToSend[1]=u.i;
  GWEN_Tag16_WriteTagToBuffer(AQH_MSGDATA_MULTIDATA_TAGS_DATA, (const uint8_t*) arrayToSend, 2*sizeof(uint64_t), buf);

  msg=AQH_IpcMessage_new(AQH_IPC_PROTOCOL_DATA_ID, AQH_IPC_PROTOCOL_DATA_VERSION, code, msgId, refMsgId,
                         GWEN_Buffer_GetUsedBytes(buf), (const uint8_t*) GWEN_Buffer_GetStart(buf));
  GWEN_Buffer_free(buf);
  return msg;
}



uint32_t AQH_IpcdMessageMultiData_GetFlags(const GWEN_TAG16_LIST *tagList)
{
  return tagList?AQH_Tag16_GetTagDataAsUint32(tagList,
                                              AQH_MSGDATA_MULTIDATA_TAGS_FLAGS,
                                              AQH_MSGDATA_MULTIDATA_FLAGS_LASTMSG):AQH_MSGDATA_MULTIDATA_FLAGS_LASTMSG;
}






AQH_VALUE *AQH_IpcdMessageMultiData_ReadValue(const GWEN_TAG16_LIST *tagList)
{
  return tagList?AQH_Tag16_ReadValueFromTagList(tagList, AQH_MSGDATA_MULTIDATA_TAGS_VALUE):NULL;
}



void AQH_IpcdMessageMultiData_ReadDatapoints(const GWEN_TAG16_LIST *tagList, const uint64_t **pDataPtr, uint64_t *pNumberOfPoints)
{
  if (tagList) {
    const GWEN_TAG16 *tag;
    unsigned int numberOfPoints;
    const uint64_t *dataPoints;

    tag=GWEN_Tag16_List_FindFirstByTagType(tagList, AQH_MSGDATA_MULTIDATA_TAGS_DATA);
    numberOfPoints=(tag?GWEN_Tag16_GetTagLength(tag):0)/(2*sizeof(uint64_t));
    dataPoints=tag?((const uint64_t*) GWEN_Tag16_GetTagData(tag)):NULL;
    if (numberOfPoints>0 && dataPoints) {
      *pDataPtr=dataPoints;
      *pNumberOfPoints=numberOfPoints;
    }
  }
}



void AQH_IpcdMessageMultiData_DumpToBuffer(const AQH_MESSAGE *msg, const GWEN_TAG16_LIST *tagList, GWEN_BUFFER *dbuf, const char *sText)
{
  AQH_VALUE *value;
  const GWEN_TAG16 *tag;
  const char *valueName;
  const char *valueUnits;
  int valueType;
  unsigned int numberOfPoints=0;
  uint32_t flags;

  flags=AQH_IpcdMessageMultiData_GetFlags(tagList);
  value=tagList?AQH_IpcdMessageMultiData_ReadValue(tagList):NULL;
  valueName=value?AQH_Value_GetNameForSystem(value):NULL;
  valueUnits=value?AQH_Value_GetValueUnits(value):NULL;
  valueType=value?AQH_Value_GetValueType(value):0;

  tag=tagList?GWEN_Tag16_List_FindFirstByTagType(tagList, AQH_MSGDATA_MULTIDATA_TAGS_DATA):NULL;
  numberOfPoints=(tag?GWEN_Tag16_GetTagLength(tag):0)/(2*sizeof(uint64_t));

  GWEN_Buffer_AppendArgs(dbuf,
                         "MULTIDATA(%s) %s (code=%d, proto=%d, proto version=%d, flags=0x%08x, "
                         "name=%s, units=%s, type=%d, datapoints=%u)\n",
                         AQH_IpcdMessage_MsgTypeToChar(AQH_IpcMessage_GetCode(msg)),
			 sText?sText:"",
			 AQH_IpcMessage_GetCode(msg),
			 AQH_IpcMessage_GetProtoId(msg),
                         AQH_IpcMessage_GetProtoVersion(msg),
                         flags,
			 valueName?valueName:"<empty>",
			 valueUnits?valueUnits:"<empty>",
			 valueType,
			 numberOfPoints);
  AQH_Value_free(value);
}





